@@ -1,8 +1,9 @@
 #!/bin/sh
 server=rabbitmq-server
 ctl=rabbitmqctl
-plugins=rabbitmq-plugins
-delay=3
+delay=5
+
+echo "[Configuration] $(eval cat $RABBITMQ_CONFIG_FILE.config)"
 
 echo '[Configuration] Starting RabbitMQ in detached mode.'
 
@@ -12,10 +13,6 @@ echo "[Configuration] Waiting $delay seconds for RabbitMQ to start."
 
 sleep $delay
 
-echo '*** Enabling plugins ***'
-$plugins enable --online rabbitmq_management
-$plugins enable --online rabbitmq_consistent_hash_exchange
-
 echo '*** Creating users ***'
 $ctl add_user bunny_gem bunny_password
 $ctl add_user bunny_reader reader_password
