@@ -182,6 +182,9 @@ module Bunny
       @publishing_mutex = @connection.mutex_impl.new
       @consumer_mutex   = @connection.mutex_impl.new
 
+      @queue_mutex    = @connection.mutex_impl.new
+      @exchange_mutex = @connection.mutex_impl.new
+
       @unconfirmed_set_mutex = @connection.mutex_impl.new
 
       self.reset_continuations
@@ -230,6 +233,9 @@ module Bunny
     # {Bunny::Queue}, {Bunny::Exchange} and {Bunny::Consumer} instances.
     # @api public
     def close
+      # see bunny#528
+      raise_if_no_longer_open!
+
       @connection.close_channel(self)
       @status = :closed
       @work_pool.shutdown
@@ -305,7 +311,7 @@ module Bunny
     # @see http://rubybunny.info/articles/extensions.html RabbitMQ Extensions to AMQP 0.9.1 guide
     # @api public
     def fanout(name, opts = {})
-      Exchange.new(self, :fanout, name, opts)
+      find_exchange(name) || Exchange.new(self, :fanout, name, opts)
     end
 
     # Declares a direct exchange or looks it up in the cache of previously
@@ -323,7 +329,7 @@ module Bunny
     # @see http://rubybunny.info/articles/extensions.html RabbitMQ Extensions to AMQP 0.9.1 guide
     # @api public
     def direct(name, opts = {})
-      Exchange.new(self, :direct, name, opts)
+      find_exchange(name) || Exchange.new(self, :direct, name, opts)
     end
 
     # Declares a topic exchange or looks it up in the cache of previously
@@ -341,7 +347,7 @@ module Bunny
     # @see http://rubybunny.info/articles/extensions.html RabbitMQ Extensions to AMQP 0.9.1 guide
     # @api public
     def topic(name, opts = {})
-      Exchange.new(self, :topic, name, opts)
+      find_exchange(name) || Exchange.new(self, :topic, name, opts)
     end
 
     # Declares a headers exchange or looks it up in the cache of previously
@@ -359,7 +365,7 @@ module Bunny
     # @see http://rubybunny.info/articles/extensions.html RabbitMQ Extensions to AMQP 0.9.1 guide
     # @api public
     def headers(name, opts = {})
-      Exchange.new(self, :headers, name, opts)
+      find_exchange(name) || Exchange.new(self, :headers, name, opts)
     end
 
     # Provides access to the default exchange
@@ -599,7 +605,12 @@ module Bunny
       # we implement them). So we return a triple of nils immediately which apps should be
       # able to handle anyway as "got no message, no need to act". MK.
       last_basic_get_response = if @connection.open?
-                                  wait_on_basic_get_continuations
+                                  begin
+                                    wait_on_basic_get_continuations
+                                  rescue Timeout::Error => e
+                                    raise_if_continuation_resulted_in_a_channel_error!
+                                    raise e
+                                  end
                                 else
                                   [nil, nil, nil]
                                 end
@@ -1381,7 +1392,7 @@ module Bunny
     # @see #nacked_set
     # @see http://rubybunny.info/articles/extensions.html RabbitMQ Extensions guide
     # @api public
-    def confirm_select(callback=nil)
+    def confirm_select(callback = nil)
       raise_if_no_longer_open!
 
       if @next_publish_seq_no == 0
@@ -1516,7 +1527,7 @@ module Bunny
     #
     # @api plugin
     def recover_exchanges
-      @exchanges.values.dup.each do |x|
+      @exchange_mutex.synchronize { @exchanges.values }.each do |x|
         x.recover_from_network_failure
       end
     end
@@ -1526,7 +1537,7 @@ module Bunny
     #
     # @api plugin
     def recover_queues
-      @queues.values.dup.each do |q|
+      @queue_mutex.synchronize { @queues.values }.each do |q|
         @logger.debug { "Recovering queue #{q.name}" }
         q.recover_from_network_failure
       end
@@ -1541,7 +1552,8 @@ module Bunny
         @work_pool = ConsumerWorkPool.new(@work_pool.size, @work_pool.abort_on_exception)
         @work_pool.start
       end
-      @consumers.values.dup.each do |c|
+
+      @consumer_mutex.synchronize { @consumers.values }.each do |c|
         c.recover_from_network_failure
       end
     end
@@ -1861,37 +1873,37 @@ module Bunny
 
     # @private
     def deregister_queue(queue)
-      @queues.delete(queue.name)
+      @queue_mutex.synchronize { @queues.delete(queue.name) }
     end
 
     # @private
     def deregister_queue_named(name)
-      @queues.delete(name)
+      @queue_mutex.synchronize { @queues.delete(name) }
     end
 
     # @private
     def register_queue(queue)
-      @queues[queue.name] = queue
+      @queue_mutex.synchronize { @queues[queue.name] = queue }
     end
 
     # @private
     def find_queue(name)
-      @queues[name]
+      @queue_mutex.synchronize { @queues[name] }
     end
 
     # @private
     def deregister_exchange(exchange)
-      @exchanges.delete(exchange.name)
+      @exchange_mutex.synchronize { @exchanges.delete(exchange.name) }
     end
 
     # @private
     def register_exchange(exchange)
-      @exchanges[exchange.name] = exchange
+      @exchange_mutex.synchronize { @exchanges[exchange.name] = exchange }
     end
 
     # @private
     def find_exchange(name)
-      @exchanges[name]
+      @exchange_mutex.synchronize { @exchanges[name] }
     end
 
     protected
@@ -1931,7 +1943,13 @@ module Bunny
 
     # @private
     def raise_if_no_longer_open!
-      raise ChannelAlreadyClosed.new("cannot use a channel that was already closed! Channel id: #{@id}", self) if closed?
+      if closed?
+        if @last_channel_error
+          raise ChannelAlreadyClosed.new("cannot use a closed channel! Channel id: #{@id}, closed due to a server-reported channel error: #{@last_channel_error.message}", self)
+        else
+          raise ChannelAlreadyClosed.new("cannot use a closed channel! Channel id: #{@id}", self)
+        end
+      end
     end
 
     # @private
@@ -1971,7 +1989,7 @@ module Bunny
     def guarding_against_stale_delivery_tags(tag, &block)
       case tag
         # if a fixnum was passed, execute unconditionally. MK.
-      when Fixnum then
+      when Integer then
         block.call
         # versioned delivery tags should be checked to avoid
         # sending out stale (invalid) tags after channel was reopened
