@@ -9,8 +9,21 @@ module Bunny
   module Socket
     attr_accessor :options
 
-    READ_RETRY_EXCEPTION_CLASSES = [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable]
-    WRITE_RETRY_EXCEPTION_CLASSES = [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable]
+    READ_RETRY_EXCEPTION_CLASSES = if defined?(IO::EAGAINWaitReadable)
+                                     # Ruby 2.1+
+                                     [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable,
+                                      IO::EAGAINWaitReadable, IO::EWOULDBLOCKWaitReadable]
+                                   else
+                                     # 2.0
+                                     [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable]
+                                   end
+    WRITE_RETRY_EXCEPTION_CLASSES = if defined?(IO::EAGAINWaitWritable)
+                                      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable,
+                                       IO::EAGAINWaitWritable, IO::EWOULDBLOCKWaitWritable]
+                                    else
+                                      # 2.0
+                                      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable]
+                                    end
 
     def self.open(host, port, options = {})
       socket = ::Socket.tcp(host, port, nil, nil,
