@@ -4,7 +4,7 @@ require "monitor"
 
 begin
   require "openssl"
-rescue LoadError => le
+rescue LoadError => _le
   $stderr.puts "Could not load OpenSSL"
 end
 
@@ -26,7 +26,7 @@ module Bunny
     DEFAULT_WRITE_TIMEOUT = 30.0
 
     attr_reader :session, :host, :port, :socket, :connect_timeout, :read_timeout, :write_timeout, :disconnect_timeout
-    attr_reader :tls_context
+    attr_reader :tls_context, :verify_peer, :tls_ca_certificates, :tls_certificate_path, :tls_key_path
 
     attr_writer :read_timeout
 
@@ -92,7 +92,7 @@ module Bunny
     end
 
     def connected?
-      :not_connected == @status && open?
+      :connected == @status && open?
     end
 
     def configure_socket(&block)
@@ -233,17 +233,13 @@ module Bunny
     # Exposed primarily for Bunny::Channel
     # @private
     def read_next_frame(opts = {})
-      header    = read_fully(7)
-      # TODO: network issues here will sometimes cause
-      #       the socket method return an empty string. We need to log
-      #       and handle this better.
-      # type, channel, size = begin
-      #                         AMQ::Protocol::Frame.decode_header(header)
-      #                       rescue AMQ::Protocol::EmptyResponseError => e
-      #                         puts "Got AMQ::Protocol::EmptyResponseError, header is #{header.inspect}"
-      #                       end
+      header              = read_fully(7)
       type, channel, size = AMQ::Protocol::Frame.decode_header(header)
-      payload   = read_fully(size)
+      payload             = if size > 0
+                              read_fully(size)
+                            else
+                              ''
+                            end
       frame_end = read_fully(1)
 
       # 1) the size is miscalculated
@@ -263,7 +259,7 @@ module Bunny
           :connect_timeout => timeout)
 
         true
-      rescue SocketError, Timeout::Error => e
+      rescue SocketError, Timeout::Error => _e
         false
       ensure
         s.close if s
@@ -302,23 +298,27 @@ module Bunny
     protected
 
     def tls_enabled?(opts)
-      return opts[:tls] unless opts[:tls].nil?
-      return opts[:ssl] unless opts[:ssl].nil?
+      return !!opts[:tls] unless opts[:tls].nil?
+      return !!opts[:ssl] unless opts[:ssl].nil?
       (opts[:port] == AMQ::Protocol::TLS_PORT) || false
     end
 
+    def tls_ca_certificates_paths_from(opts)
+      Array(opts[:cacertfile] || opts[:tls_ca_certificates] || opts[:ssl_ca_certificates])
+    end
+
     def tls_certificate_path_from(opts)
-      opts[:tls_cert] || opts[:ssl_cert] || opts[:tls_cert_path] || opts[:ssl_cert_path] || opts[:tls_certificate_path] || opts[:ssl_certificate_path]
+      opts[:certfile] || opts[:tls_cert] || opts[:ssl_cert] || opts[:tls_cert_path] || opts[:ssl_cert_path] || opts[:tls_certificate_path] || opts[:ssl_certificate_path]
     end
 
     def tls_key_path_from(opts)
-      opts[:tls_key] || opts[:ssl_key] || opts[:tls_key_path] || opts[:ssl_key_path]
+      opts[:keyfile] || opts[:tls_key] || opts[:ssl_key] || opts[:tls_key_path] || opts[:ssl_key_path]
     end
 
     def tls_certificate_from(opts)
       begin
         read_client_certificate!
-      rescue MissingTLSCertificateFile => e
+      rescue MissingTLSCertificateFile => _e
         inline_client_certificate_from(opts)
       end
     end
@@ -326,7 +326,7 @@ module Bunny
     def tls_key_from(opts)
       begin
         read_client_key!
-      rescue MissingTLSKeyFile => e
+      rescue MissingTLSKeyFile => _e
         inline_client_key_from(opts)
       end
     end
@@ -341,7 +341,7 @@ module Bunny
     end
 
     def prepare_tls_context(opts)
-      if (opts[:verify_ssl] || opts[:verify_peer]).nil?
+      if opts.values_at(:verify_ssl, :verify_peer, :verify).all?(&:nil?)
         opts[:verify_peer] = true
       end
 
@@ -353,17 +353,34 @@ module Bunny
       @tls_key               = tls_key_from(opts)
       @tls_certificate_store = opts[:tls_certificate_store]
 
-      @tls_ca_certificates   = opts.fetch(:tls_ca_certificates, default_tls_certificates)
-      @verify_peer           = (opts[:verify_ssl] || opts[:verify_peer])
+      @tls_ca_certificates   = tls_ca_certificates_paths_from(opts)
+      @tls_ca_certificates   = default_tls_certificates if @tls_ca_certificates.empty?
+      @verify_peer           = as_boolean(opts[:verify_ssl] || opts[:verify_peer] || opts[:verify])
 
       @tls_context = initialize_tls_context(OpenSSL::SSL::SSLContext.new, opts)
     end
 
+    def as_boolean(val)
+      case val
+      when true    then true
+      when false   then false
+      when "true"  then true
+      when "false" then false
+      else
+        !!val
+      end
+    end
+
     def wrap_in_tls_socket(socket)
       raise ArgumentError, "cannot wrap nil into TLS socket, @tls_context is nil. This is a Bunny bug." unless socket
       raise "cannot wrap a socket into TLS socket, @tls_context is nil. This is a Bunny bug." unless @tls_context
 
       s = Bunny::SSLSocketImpl.new(socket, @tls_context)
+
+      # always set the SNI server name if possible since RFC 3546 and RFC 6066 both state
+      # that TLS clients supporting the extensions can talk to TLS servers that do not
+      s.hostname = @host if s.respond_to?(:hostname)
+
       s.sync_close = true
       s
     end
@@ -419,7 +436,7 @@ certificate, connection upgrade will fail!
       if !@verify_peer
         @logger.warn <<-MSG
 Using TLS but peer hostname verification is disabled. This is convenient for local development
-but prone man-in-the-middle attacks. Please set :verify_peer => true in production!
+but prone to man-in-the-middle attacks. Please set verify_peer: true in production!
         MSG
       end
 
@@ -450,9 +467,9 @@ but prone man-in-the-middle attacks. Please set :verify_peer => true in producti
       cert_files = []
       cert_inlines = []
       certs.each do |cert|
-        # if it starts with / then it's a file path that may or may not
-        # exists (e.g. a default OpenSSL path). MK.
-        if File.readable?(cert) || cert =~ /^\//
+        # if it starts with / or C:/ then it's a file path that may or may not
+        # exist (e.g. a default OpenSSL path). MK.
+        if File.readable?(cert) || cert =~ /^([a-z]:?)?\//i
           cert_files.push(cert)
         else
           cert_inlines.push(cert)
