@@ -1,5 +1,9 @@
 require "spec_helper"
 
+def local_hostname
+  ENV.fetch("BUNNY_RABBITMQ_HOSTNAME", "127.0.0.1")
+end
+
 describe Bunny::Session do
   let(:port)     { AMQ::Protocol::DEFAULT_PORT }
   let(:username) { "guest" }
@@ -78,7 +82,7 @@ describe Bunny::Session do
   unless ENV["CI"]
     context "initialized with TCP connection timeout = 5" do
       it "successfully connects" do
-        conn = described_class.new(:connection_timeout => 5)
+        conn = described_class.new(connection_timeout: 5)
         conn.start
         expect(conn).to be_connected
 
@@ -95,14 +99,14 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :host => 127.0.0.1" do
+    context "initialized with hostname: 127.0.0.1" do
       after :each do
         subject.close if subject.open?
       end
 
       let(:host)  { "127.0.0.1" }
       subject do
-        described_class.new(:host => host)
+        described_class.new(hostname: host)
       end
 
       it "uses hostname = 127.0.0.1" do
@@ -119,13 +123,13 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :hostname => localhost" do
+    context "initialized with hostname: localhost" do
       after :each do
         subject.close if subject.open?
       end
 
       let(:host)    { "localhost" }
-      let(:subject) { described_class.new(:hostname => host) }
+      let(:subject) { described_class.new(hostname: host) }
 
       it "uses hostname = localhost" do
         expect(subject.host).to eq host
@@ -142,14 +146,14 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :hosts => [...]" do
+    context "initialized with a list of hosts" do
       after :each do
         subject.close if subject.open?
       end
 
       let(:host)    { "192.168.1.10" }
       let(:hosts)   { [host] }
-      let(:subject) { described_class.new(:hosts => hosts) }
+      let(:subject) { described_class.new(hosts: hosts) }
 
       it "uses hostname = 192.168.1.10" do
         expect(subject.host).to eq host
@@ -166,7 +170,7 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :addresses => [...]" do
+    context "initialized with a list of addresses" do
       after :each do
         subject.close if subject.open?
       end
@@ -175,7 +179,7 @@ describe Bunny::Session do
       let(:port)      { 5673 }
       let(:address)   { "#{host}:#{port}" }
       let(:addresses) { [address] }
-      let(:subject)   { described_class.new(:addresses => addresses) }
+      let(:subject)   { described_class.new(addresses: addresses) }
 
       it "uses hostname = 192.168.1.10" do
         expect(subject.host).to eq host
@@ -192,7 +196,7 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :addresses => [...] with quoted IPv6 hostnames" do
+    context "initialized with addresses: [...] with quoted IPv6 hostnames" do
       after :each do
         subject.close if subject.open?
       end
@@ -201,7 +205,7 @@ describe Bunny::Session do
       let(:port)      { 5673 }
       let(:address)   { "#{host}:#{port}" }
       let(:addresses) { [address] }
-      let(:subject)   { described_class.new(:addresses => addresses) }
+      let(:subject)   { described_class.new(addresses: addresses) }
 
       it "uses correct hostname" do
         expect(subject.host).to eq host
@@ -218,7 +222,7 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :addresses => [...] with quoted IPv6 hostnames without ports" do
+    context "initialized with addresses: [...] with quoted IPv6 hostnames without ports" do
       after :each do
         subject.close if subject.open?
       end
@@ -226,7 +230,7 @@ describe Bunny::Session do
       let(:host)      { "[2001:db8:85a3:8d3:1319:8a2e:370:7348]" }
       let(:address)   { host }
       let(:addresses) { [address] }
-      let(:subject)   { described_class.new(:addresses => addresses) }
+      let(:subject)   { described_class.new(addresses: addresses) }
 
       it "uses correct hostname" do
         expect(subject.host).to eq host
@@ -243,7 +247,7 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :addresses => [...] with an quoted IPv6 hostnames" do
+    context "initialized with addresses: [...] with an quoted IPv6 hostnames" do
       after :each do
         subject.close if subject.open?
       end
@@ -252,7 +256,7 @@ describe Bunny::Session do
       let(:port)      { 5673 }
       let(:address)   { "#{host}:#{port}" }
       let(:addresses) { [address] }
-      let(:subject)   { described_class.new(:addresses => addresses) }
+      let(:subject)   { described_class.new(addresses: addresses) }
 
       it "fails to correctly parse the host (and emits a warning)" do
         expect(subject.host).to eq "2001"
@@ -282,8 +286,7 @@ describe Bunny::Session do
 
       it "logs a warning when there is a single host and an array" do
         described_class.new(addresses: [address], host: host, logger: logger)
-        expect(io.string).to include 'WARN -- : The connection options contain '\
-          'both a host and an array of hosts, the single host is ignored.'
+        expect(io.string).to match(/both a host and an array of hosts/)
       end
 
       it "converts hosts in addresses to addresses" do
@@ -295,13 +298,13 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :channel_max => 4096" do
+    context "initialized with channel_max: 4096" do
       after :each do
         subject.close if subject.open?
       end
 
       let(:channel_max) { 1024 }
-      let(:subject)     { described_class.new(:channel_max => channel_max) }
+      let(:subject)     { described_class.new(channel_max: channel_max) }
 
       # this assumes RabbitMQ has no lower value configured. In 3.2
       # it is 0 (no limit) by default and 1024 is still a fairly low value
@@ -314,15 +317,15 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :ssl => true" do
+    context "initialized with ssl: true" do
       let(:subject) do
-        described_class.new(:user     => "bunny_gem",
-          :password => "bunny_password",
-          :vhost    => "bunny_testbed",
-          :ssl                   => true,
-          :ssl_cert              => "spec/tls/client_cert.pem",
-          :ssl_key               => "spec/tls/client_key.pem",
-          :ssl_ca_certificates   => ["./spec/tls/cacert.pem"])
+        described_class.new(username: "bunny_gem",
+          password: "bunny_password",
+          vhost: "bunny_testbed",
+          ssl:                 true,
+          ssl_cert:            "spec/tls/client_certificate.pem",
+          ssl_key:             "spec/tls/client_key.pem",
+          ssl_ca_certificates: ["./spec/tls/ca_certificate.pem"])
       end
 
       it "uses TLS port" do
@@ -330,15 +333,15 @@ describe Bunny::Session do
       end
     end
 
-    context "initialized with :tls => true" do
+    context "initialized with tls: true" do
       let(:subject) do
-        described_class.new(:user     => "bunny_gem",
-          :password => "bunny_password",
-          :vhost    => "bunny_testbed",
-          :tls                   => true,
-          :tls_cert              => "spec/tls/client_certificate.pem",
-          :tls_key               => "spec/tls/client_key.pem",
-          :tls_ca_certificates   => ["./spec/tls/ca_certificate.pem"])
+        described_class.new(username: "bunny_gem",
+          password: "bunny_password",
+          vhost: "bunny_testbed",
+          tls: true,
+          tls_cert: "spec/tls/client_certificate.pem",
+          tls_key: "spec/tls/client_key.pem",
+          tls_ca_certificates: ["./spec/tls/ca_certificate.pem"])
       end
 
       it "uses TLS port" do
@@ -347,7 +350,7 @@ describe Bunny::Session do
     end
   end
 
-  context "initialized with :host => 127.0.0.1 and non-default credentials" do
+  context "initialized with hostname: 127.0.0.1 and non-default credentials" do
     after :each do
       subject.close if subject.open?
     end
@@ -359,7 +362,7 @@ describe Bunny::Session do
     let(:vhost)    { "bunny_testbed" }
 
     subject do
-      described_class.new(:hostname => host, :username => username, :password => password, :virtual_host => vhost)
+      described_class.new(hostname: host, username: username, password: password, virtual_host: vhost)
     end
 
     it "successfully connects" do
@@ -399,7 +402,7 @@ describe Bunny::Session do
     end
   end
 
-  context "initialized with :host => 127.0.0.1 and non-default credentials (take 2)" do
+  context "initialized with hostname: 127.0.0.1 and non-default credentials (take 2)" do
     after :each do
       subject.close if subject.open?
     end
@@ -411,7 +414,7 @@ describe Bunny::Session do
     let(:vhost)    { "bunny_testbed" }
 
     subject do
-      described_class.new(:hostname => host, :user => username, :pass => password, :vhost => vhost)
+      described_class.new(hostname: host, username: username, password: password, vhost: vhost)
     end
 
     it "successfully connects" do
@@ -446,7 +449,7 @@ describe Bunny::Session do
     end
   end
 
-  context "initialized with :host => 127.0.0.1 and non-default credentials (take 2)" do
+  context "initialized with hostname: 127.0.0.1 and non-default credentials (take 2)" do
     after :each do
       subject.close if subject.open?
     end
@@ -459,7 +462,7 @@ describe Bunny::Session do
     let(:interval) { 1 }
 
     subject do
-      described_class.new(:hostname => host, :user => username, :pass => password, :vhost => vhost, :heartbeat_interval => interval)
+      described_class.new(hostname: host, username: username, password: password, vhost: vhost, heartbeat_interval: interval)
     end
 
     it "successfully connects" do
@@ -482,7 +485,7 @@ describe Bunny::Session do
     end
   end
 
-  context "initialized with :host => 127.0.0.1 and INVALID credentials" do
+  context "initialized with hostname: 127.0.0.1 and INVALID credentials" do
     let(:host)     { "127.0.0.1" }
     # see ./bin/ci/before_build
     let(:username) { "bunny_gem#{Time.now.to_i}" }
@@ -490,7 +493,7 @@ describe Bunny::Session do
     let(:vhost)    { "___sd89aysd98789" }
 
     subject do
-      described_class.new(:hostname => host, :user => username, :pass => password, :vhost => vhost)
+      described_class.new(hostname: host, username: username, password: password, vhost: vhost)
     end
 
     it "fails to connect" do
@@ -511,14 +514,14 @@ describe Bunny::Session do
   context "initialized with unreachable host or port" do
     it "fails to connect" do
       expect do
-        c = described_class.new(:port => 38000)
+        c = described_class.new(port: 38000)
         c.start
       end.to raise_error(Bunny::TCPConnectionFailed)
     end
 
     it "is not connected" do
       begin
-        c = described_class.new(:port => 38000)
+        c = described_class.new(port: 38000)
         c.start
       rescue Bunny::TCPConnectionFailed => e
         true
@@ -529,7 +532,7 @@ describe Bunny::Session do
 
     it "is not open" do
       begin
-        c = described_class.new(:port => 38000)
+        c = described_class.new(port: 38000)
         c.start
       rescue Bunny::TCPConnectionFailed => e
         true
@@ -544,7 +547,7 @@ describe Bunny::Session do
     let(:logger)  { ::Logger.new(io) }
 
     it "uses provided logger" do
-      conn = described_class.new(:logger => logger)
+      conn = described_class.new(logger: logger)
       conn.start
 
       expect(io.string.length).to be > 100
@@ -554,7 +557,7 @@ describe Bunny::Session do
 
     it "doesn't reassign the logger's progname attribute" do
       expect(logger).not_to receive(:progname=)
-      described_class.new(:logger => logger)
+      described_class.new(logger: logger)
     end
   end
 end
