@@ -2,7 +2,7 @@ require "spec_helper"
 
 describe Bunny::Channel do
   let(:connection) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
@@ -20,18 +20,18 @@ describe Bunny::Channel do
       ch = connection.create_channel
       t  = Thread.new do
         ch2 = connection.create_channel
-        q   = ch2.queue(queue_name, :auto_delete => true)
+        q   = ch2.queue(queue_name, auto_delete: true)
 
-        q.subscribe(:on_cancellation => Proc.new { cancelled = true })
+        q.subscribe(on_cancellation: Proc.new { cancelled = true })
       end
       t.abort_on_exception = true
 
       sleep 0.5
       x = ch.default_exchange
-      x.publish("abc", :routing_key => queue_name)
+      x.publish("abc", routing_key: queue_name)
 
       sleep 0.5
-      ch.queue(queue_name, :auto_delete => true).delete
+      ch.queue(queue_name, auto_delete: true).delete
 
       sleep 0.5
       expect(cancelled).to eq true
@@ -60,7 +60,7 @@ describe Bunny::Channel do
       ch = connection.create_channel
       t  = Thread.new do
         ch2 = connection.create_channel
-        q   = ch2.queue(queue_name, :auto_delete => true)
+        q   = ch2.queue(queue_name, auto_delete: true)
 
         consumer = ExampleConsumer.new(ch2, q, "")
         q.subscribe_with(consumer)
@@ -69,10 +69,10 @@ describe Bunny::Channel do
 
       sleep 0.5
       x = ch.default_exchange
-      x.publish("abc", :routing_key => queue_name)
+      x.publish("abc", routing_key: queue_name)
 
       sleep 0.5
-      ch.queue(queue_name, :auto_delete => true).delete
+      ch.queue(queue_name, auto_delete: true).delete
 
       sleep 0.5
       expect(consumer).to be_cancelled
@@ -86,7 +86,7 @@ describe Bunny::Channel do
   context "with consumer re-registration" do
     class ExampleConsumerThatReregisters < Bunny::Consumer
       def handle_cancellation(_)
-        @queue = @channel.queue("basic.consume.after_cancellation", :auto_delete => true)
+        @queue = @channel.queue("basic.consume.after_cancellation", auto_delete: true)
         @channel.basic_consume_with(self)
       end
     end
@@ -100,7 +100,7 @@ describe Bunny::Channel do
       ch = connection.create_channel
       t  = Thread.new do
         ch2 = connection.create_channel
-        q   = ch2.queue(queue_name, :auto_delete => true)
+        q   = ch2.queue(queue_name, auto_delete: true)
 
         consumer = ExampleConsumerThatReregisters.new(ch2, q, "")
         consumer.on_delivery do |_, _, payload|
@@ -112,14 +112,14 @@ describe Bunny::Channel do
 
       sleep 0.5
       x = ch.default_exchange
-      x.publish("abc", :routing_key => queue_name)
+      x.publish("abc", routing_key: queue_name)
 
       sleep 0.5
-      ch.queue(queue_name, :auto_delete => true).delete
+      ch.queue(queue_name, auto_delete: true).delete
 
-      x.publish("abc", :routing_key => queue_name)
+      x.publish("abc", routing_key: queue_name)
       sleep 0.5
-      q = ch.queue("basic.consume.after_cancellation", :auto_delete => true)
+      q = ch.queue("basic.consume.after_cancellation", auto_delete: true)
       expect(xs).to eq ["abc"]
 
       ch.close
