@@ -2,31 +2,31 @@ require "spec_helper"
 
 describe "A message that is proxied by multiple intermediate consumers" do
   let(:c1) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
 
   let(:c2) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
 
   let(:c3) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
 
   let(:c4) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
 
   let(:c5) do
-    c = Bunny.new(:user => "bunny_gem", :password => "bunny_password", :vhost => "bunny_testbed")
+    c = Bunny.new(username: "bunny_gem", password: "bunny_password", vhost: "bunny_testbed")
     c.start
     c
   end
@@ -45,25 +45,25 @@ describe "A message that is proxied by multiple intermediate consumers" do
     xs  = []
 
     ch1 = c1.create_channel
-    q1  = ch1.queue("", :exclusive => true)
+    q1  = ch1.queue("", exclusive: true)
     q1.subscribe do |_, _, payload|
       xs << payload
     end
 
     ch2 = c2.create_channel
-    q2  = ch2.queue("", :exclusive => true)
+    q2  = ch2.queue("", exclusive: true)
     q2.subscribe do |_, _, payload|
       q1.publish(payload)
     end
 
     ch3 = c3.create_channel
-    q3  = ch2.queue("", :exclusive => true)
+    q3  = ch2.queue("", exclusive: true)
     q3.subscribe do |_, _, payload|
       q2.publish(payload)
     end
 
     ch4 = c4.create_channel
-    q4  = ch2.queue("", :exclusive => true)
+    q4  = ch2.queue("", exclusive: true)
     q4.subscribe do |_, _, payload|
       q3.publish(payload)
     end
@@ -72,7 +72,7 @@ describe "A message that is proxied by multiple intermediate consumers" do
     x   = ch5.default_exchange
 
     n.times do |i|
-      x.publish("msg #{i}", :routing_key => q4.name)
+      x.publish("msg #{i}", routing_key: q4.name)
     end
 
     t = n / 1000 * 3.0
